#!/usr/bin/perl

# Monacor DMT-3000 RS - Perl data acquisition for Linux
# (C) 2000 by Hans Schou http://www.schou.dk
# License: AFL/Public Domain

# Usage:
#  ./setbaud.sh
#  cat `cat DEVICE` | ./monacor.pl

# Tabstop = 2

use POSIX;   # gmtime

# Power calculation i.e. "m" -> 10^-3 -> 10**$pow{"m"}
@pow{"m"," ","K","M"} = (-3,0,3,6);

$n = 0;      # line counter
$mean = 0;   # mean calculation
$sum = 0;    # sum of all sample values
$min = 0;    # minimum value
$max = 0;    # maximum value
$stddev = 0; # Standard deviation
$sumsqr = 0; # StdDev, temporary

# Sampling over one second
$l_n = 0;    # Local line counter
$l_sum = 0;  # Local sum
$l_now = ""; # Local (last) '$now'

$header =
"Date       Time     Num   Fnc  Value   Min   Max    Mean StdDev\n";
#2000-05-15 00:13:08 00001 ACV   0.03   0.0   0.0   0.035 0.0000

open(RAW,">>raw.big.log") || die "Can't open raw.big.log";
open(BIG,">>big.log") || die "Can't open big.log";
# open(SQL,"| psql database") || die "Can't open PostgreSQL";

# Read from <STDIN>
while (<>) {
	# Save raw output for debugging
	print RAW $_;
	# Remove trailing <LF>
	chomp;
	# Parse and split '$_'. See bottom of file for samples
	if ( /^(ACV|ACA|DCV|DCA|OHM|CNT|DIO)([\s-]\d{4})([\smMK])([VA])([0123])$/ ) {
		# Increase line counter
		$n++;
		# Calculate value with respect to 'mV' and decimal position
		$val = $2 * 10**($pow{$3}-$5);
		$sum += $val;
		# Calculate mean
		$mean = $sum / $n;
		$sumsqr += $val**2;
		# Calculate standard deviation when more than 10 samples
		if ($n > 10) {
			$stddev = sqrt( ($n*$sumsqr-$sum**2) / ($n*($n-1)) );
		}
		# If first line store min/max
		if ($n == 1) {
			print $header;
			$min = $val;
			$max = $val;
		}
		# Set min if val is lower
		if ($val < $min) {
			$min = $val;
		}
		# Set max if val is higher
		if ($val > $max) {
			$max = $val;
		}
		# Get and format date/time
		$now = strftime "%Y-%m-%d %H:%M:%S", gmtime;
		# Count and sum on local within one second
		$l_n++;
		$l_sum += $val;
		# if time (seconds) has changed, then print
		if ($now ne $l_now) {
			$val = $l_sum / $l_n;
			$line = sprintf("$now %05d $1 %6.2f %5.1f %5.1f %7.3f %06.4f\n",
				$n,$val,$min,$max,$mean,$stddev);
			# Print to <STDOUT>
			print $line;
			# Print to file
			print BIG $line;
			# print SQL "INSERT INTO monacor(tsdate,val) VALUES('$now',$val);\n";
			# Clean up and reset
			$l_now = $now;
			$l_n = 0;
			$l_sum = 0;
		}
	} else {
		print "Error: \"$_\"\n";
	}
}
# close(SQL);
close(BIG);
close(RAW);

# Sample output from Monacor DMT-3000 RS:
# ACV 0345mV1<cr>
# ACV-4174 V1<cr>
# DCV 0582 V3<cr>
# DCV-0792 V3<cr>
# DCV-0490mV1<cr>
# DCV-1654mV1<cr>
# DCV 0383 V1<cr>
# OHM 1976M3<cr>
# OHM 9904K2<cr>
# CNT 2825K3<cr>
# DIO 2825 V3<cr>
# DCA-0000mA3<cr>
# ACA 0367 A3<cr>

